SystemOrganization addCategory: #'SqSync-Shared'!
SystemOrganization addCategory: #'SqSync-Client'!
SystemOrganization addCategory: #'SqSync-Server'!
SystemOrganization addCategory: #'SqSync-Tests'!

OrderedCollection subclass: #SySharedList
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SySharedList commentStamp: 'mt 5/9/2010 17:04:27.753' prior: 0!
This is meant to be a thread-safe version of an OrderedCollection. See the "list" protocol to explore selectors which are thread-safe.!

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:52:24.122'!
add: anElement

	^ self mutex critical: [super add: anElement].! !

!SySharedList methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:40:52.25'!
mutex

	^ mutex ifNil: [mutex := Mutex new]! !

!SySharedList methodsFor: 'list' stamp: 'mt 5/9/2010 16:53:19.735'!
remove: anElement ifAbsent: aBlock
	"#remove: should also be guarded now."
	
	^ self mutex critical: [super remove: anElement ifAbsent: aBlock].! !

Object subclass: #SyNetworkDataProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyNetworkDataProcessor commentStamp: 'mt 5/9/2010 17:01:16.718' prior: 0!
This is a place holder for the "P"" in our diagram.

Current contract: NO STATE - ONLY BEHAVIOR!

!SyNetworkDataProcessor class methodsFor: 'processing' stamp: 'mt 5/9/2010 17:02:23.913'!
processDataFrom: aStream
	"Hint: Expect stream API from SocketStream. Other Streams should work as well."! !

Object subclass: #SyTcpServer
	instanceVariableNames: 'listenSocket connections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Server'!

!SyTcpServer methodsFor: 'connection open/close' stamp: 'mt 5/10/2010 10:34:31.01'!
close

	self listenSocket closeAndDestroy.
	listenSocket := nil.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 16:46:22.708'!
connections

	^ connections ifNil: [connections := SySharedList new]! !

!SyTcpServer methodsFor: 'testing' stamp: 'mt 5/9/2010 16:17:23.888'!
isListening

	^ self listenSocket isWaitingForConnection! !

!SyTcpServer methodsFor: 'connection open/close' stamp: 'mt 5/9/2010 18:22:57.243'!
listenOn: aPort

	self listenSocket listenOn: aPort backlogSize: 4.
	self startListenThread.! !

!SyTcpServer methodsFor: 'accessing - private' stamp: 'mt 5/9/2010 15:58:25.953'!
listenSocket

	^ listenSocket ifNil: [listenSocket := Socket newTCP]! !

!SyTcpServer methodsFor: 'accessing' stamp: 'mt 5/9/2010 17:12:59.484'!
port

	^ self listenSocket port! !

!SyTcpServer methodsFor: 'threads' stamp: 'mt 5/10/2010 11:08:51.265'!
startConnectionThreadFor: aTcpSocket

	[ [aTcpSocket isConnected] whileTrue: [
		aTcpSocket
			waitForDataIfClosed: [self connections remove: aTcpSocket. ^ self].
		SyNetworkDataProcessor
			processDataFrom: aTcpSocket.
	]] forkAt: 35.! !

!SyTcpServer methodsFor: 'threads' stamp: 'mt 5/10/2010 11:51:01.766'!
startListenThread
	
	[ [self isListening] whileTrue: [
		(self listenSocket waitForAcceptFor: 10)
			ifNotNilDo: [:socket |
				| con |
				con := SyTcpSocket on: socket.
				self connections add: con.
				self startConnectionThreadFor: con].
	]] forkAt: 35.! !

!SyTcpServer methodsFor: 'waiting' stamp: 'mt 5/10/2010 11:27:21.847'!
waitForNewConnectionFrom: nameOrAddressString port: aPort

	| remoteAddress |
	remoteAddress := NetNameResolver addressForName: nameOrAddressString timeout: 20.
	
	[
		self connections includes: [:tcpSocket | 
			tcpSocket remoteAddress = remoteAddress
			and: [tcpSocket remotePort = aPort]].
		Processor yield
	] whileFalse.! !

TestCase subclass: #SyCommunicationModuleTest
	instanceVariableNames: 'tcpServer tcpSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Tests'!

!SyCommunicationModuleTest class methodsFor: 'constants' stamp: 'mt 5/10/2010 11:06:45.568'!
defaultPort

	^ 22555! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 10:32:55.524'!
closeTcpServer

	(tcpServer notNil and: [tcpServer isListening])
		ifTrue: [tcpServer close].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 11:32:08.038'!
connectTcpSocketToServer

	tcpSocket := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/9/2010 15:49:58.117'!
disconnectTcpSocketFromServer

	(tcpSocket notNil and: [tcpSocket isConnected])
		ifTrue: [tcpSocket close. tcpSocket := nil].! !

!SyCommunicationModuleTest methodsFor: 'support' stamp: 'mt 5/10/2010 11:39:41.976'!
startTcpServer

	tcpServer ifNil: [tcpServer := SyTcpServer new].
	tcpServer isListening
		ifFalse: [tcpServer listenOn: self class defaultPort].! !

!SyCommunicationModuleTest methodsFor: 'running' stamp: 'mt 5/10/2010 10:45:11.577'!
tearDown

	super tearDown.
	
	self
		disconnectTcpSocketFromServer;
		closeTcpServer.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'mt 5/9/2010 17:12:15.94'!
testTcpServerIncomingConnections

	| anotherClient |
	self
		startTcpServer;
		should: tcpServer connections isEmpty;
		connectTcpSocketToServer;
		should: tcpServer connections size = 1.
	
	anotherClient := SyTcpSocket
		connectToHost: '127.0.0.1'
		port: self class defaultPort.
		
	self
		should: tcpServer connections size = 2;
		disconnectTcpSocketFromServer;
		should: tcpServer connections size = 1.
	
	anotherClient close.
	
	self should: tcpServer connections isEmpty.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'mt 5/9/2010 15:54:11.647'!
testTcpServerShutdown
	
	self
		startTcpServer;
		closeTcpServer;
		shouldnt: tcpServer isListening;
		should: tcpServer connections isEmpty.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'mt 5/10/2010 11:12:42.799'!
testTcpServerStartup

	| sut |
	sut := SyTcpServer new.
	
	self shouldnt: sut isListening.
	
	sut listenOn: self class defaultPort.
	
	self
		should: sut isListening;
		should: sut port = self class defaultPort;
		should: sut connections isEmpty.
		
	sut close.! !

!SyCommunicationModuleTest methodsFor: 'tests' stamp: 'mt 5/10/2010 11:35:35.678'!
testTcpSocketConnection

	self
		connectTcpSocketToServer;
		shouldnt: tcpSocket isConnected;
		startTcpServer;
		should: tcpSocket isConnected.! !

SocketStream subclass: #SyTcpSocket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSync-Shared'!

!SyTcpSocket class methodsFor: 'instance creation' stamp: 'mt 5/10/2010 11:05:32.461'!
connectToHost: nameOrAddressString port: anInteger
	
	^ self
		openConnectionToHostNamed: nameOrAddressString
		port: anInteger! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:48.023'!
remoteAddress

	^ self socket remoteAddress! !

!SyTcpSocket methodsFor: 'accessing' stamp: 'mt 5/10/2010 11:23:53.871'!
remotePort

	^ self socket remotePort! !

!SyTcpSocket methodsFor: 'waiting' stamp: 'mt 5/9/2010 17:23:15.848'!
waitForDataIfClosed: aBlock

	^ self socket waitForDataIfClosed: aBlock! !
